from kdezero.datasets.core import Dataset
from kdezero.datasets.imagenet import ImageNet
from kdezero.datasets.mnist import MNIST
from kdezero.datasets.sincurve import SinCurve
from kdezero.datasets.spiral import Spiral
